﻿init python:
    import random
    import time
    random.seed(time.clock())

    class shooter_clock(renpy.Displayable):
        def __init__(self):
            super(renpy.Displayable,self).__init__()
            self.start = 0
            self.current = 0
        def render(self,width,height,st,at):
            self.current += 1

            renpy.redraw(self, 0)
            return renpy.Render(0, 0)

    class shooter_enemy(renpy.Displayable):
        def __init__(self, images, x, y):
            super(renpy.Displayable,self).__init__()
            self.image = random.choice(images)
            self.x = x
            self.y = y
            self.z = .01
            self.stat = "_idle"
            self.time = 0
            self.parent = None
        def change(self, stat):
            self.stat = stat
            self.time = 0
        def shoot(self):
            renpy.play("shooter/gun_1.ogg", "sound")
            self.change("_shooting")
            self.parent.got_shot()
        def render(self,width,height,st,at):
            if self.z < 1:
                self.z += .1
            else:
                self.z = 1
            self.time += 1
            if self.time > 200 and self.stat == "_idle":
                self.shoot()
            elif self.time > 100 and self.stat == "_shooting":
                self.change("_idle")
            elif self.time > 100 and self.stat == "_dead":
                self.change("hidden")
            img = self.image + self.stat
            if self.stat == "hidden":
                return renpy.Render(0, 0)
            t = Transform(img, xanchor=0.5, yanchor=0.5, yzoom = self.z)
            child_render = renpy.render(t, width, height, st, at)
            cw, ch = child_render.get_size()

            rv = renpy.Render(cw, ch)
            rv.blit(child_render, (0, 0))

            renpy.redraw(self, 0)
            return rv

    class shooter_handler:
        def __init__(self, enemies):
            self.enemies = enemies
            self.active_enemies = []
            self.stat = "start"
            self.clock = shooter_clock()
            self.health = 5
            self.bullets = 6
            self.kills = 0
        def start(self):
            self.stat = "playing"
        def restart(self):
            self.kills = 0
            self.health = 5
            self.bullets = 6
            self.active_enemies = []
            self.stat = "playing"
        def spawn(self):
            for i in self.active_enemies:
                if i.stat == "hidden":
                    self.active_enemies.remove(i)
            if len(self.active_enemies) < 3:
                e = random.choice(self.enemies)
                enemy = shooter_enemy(*e)
                for i in self.active_enemies:
                    if i.x == enemy.x:
                        if i.y == enemy.y:
                            return
                enemy.parent = self
                self.active_enemies.append(enemy)
        def shoot(self, enemy):
            if self.bullets > 0:
                self.bullets -= 1
                renpy.play("shooter/gun_2.ogg", "sound")
                if not enemy:
                    return
                if not enemy.stat == "_dead":
                    renpy.play("shooter/kill.ogg", "sfx1")
                    enemy.change("_dead")
                    self.kills += 1
            else:
                renpy.play("shooter/empty.ogg", "sound")
        def got_shot(self):
            if self.stat == "playing":
                self.health -= 1
                if self.health < 1:
                    self.stat = "dead"
                    renpy.play("shooter/hurt.ogg", "sfx1")
        def reload(self):
            renpy.play("shooter/reload.ogg", "sound")
            self.bullets = 6

screen shooter():
    modal True
    default g = shooter_handler([
        [["shooter_enemy_1", "shooter_enemy_2"], 298,364],
        [["shooter_enemy_1", "shooter_enemy_2"], 108,715],
        [["shooter_enemy_1", "shooter_enemy_2"], 1798,783],
        [["shooter_enemy_1", "shooter_enemy_2"], 1540,439],
        [["shooter_enemy_1", "shooter_enemy_2"], 952,366],
    ])
    add g.clock
    button:
        background None
        action Function(g.shoot, None)
    fixed:
        xysize 1920,1080
        for i in g.active_enemies:
            button:
                padding .0,.0 pos i.x,i.y background None anchor .5,1.0
                add i
                action Function(g.shoot, i)

    if g.stat == "start":
        button:
            background "#0009" padding 20,20 align .5,.5
            text "Start"
            action Function(g.start)
    elif g.stat == "playing":
        timer 1 repeat True action Function(g.spawn)
        hbox:
            align .0,1.0 spacing 10
            for i in range(g.health):
                frame:
                    xysize 20,60 background "#ef305d"
        if g.bullets > 0:
            hbox:
                align .5,1.0 spacing 10
                for i in range(g.bullets):
                    frame:
                        xysize 20,60 background "#fff"
        else:
            button:
                align .5,1.0 background "#0009"
                text "Reload"
                action Function(g.reload)
        key "K_SPACE" action Function(g.reload)
    elif g.stat == "dead":
        hbox:
            align .5,.5 spacing 10
            button:
                background "#0009" padding 20,20
                text "Restart"
                action Function(g.restart)
            button:
                background "#0009" padding 20,20
                text "Done"
                action Return(g.kills)


    vbox:
        text str(g.kills)




image shooter_enemy_1 idle:
    "shooter_enemy_1"
    yzoom .01
    ease .2 yzoom 1
image shooter_enemy_2 idle:
    "shooter_enemy_2"
    yzoom .01
    ease .2 yzoom 1

label shooter_example:
    show shooter_background with dissolve
    call screen shooter

